@@ -59,66 +59,66 @@ use DNS::Unbound::X ();
 our ($VERSION);
 
 BEGIN {
-    $VERSION = '0.08';
+    $VERSION = '0.09';
     XSLoader::load();
 }
 
 use constant RR => {
-    A => 1,
-    AAAA => 28,
-    AFSDB => 18,
-    ANY => 255,
-    APL => 42,
-    CAA => 257,
-    CDNSKEY => 60,
-    CDS => 59,
-    CERT => 37,
-    CNAME => 5,
-    DHCID => 49,
-    DLV => 32769,
-    DNAME => 39,
-    DNSKEY => 48,
-    DS => 43,
-    HIP => 55,
-    HINFO => 13,
-    IPSECKEY => 45,
-    KEY => 25,
-    KX => 36,
-    LOC => 29,
-    MX => 15,
-    NAPTR => 35,
-    NS => 2,
-    NSEC => 47,
-    NSEC3 => 50,
+    A          => 1,
+    AAAA       => 28,
+    AFSDB      => 18,
+    ANY        => 255,
+    APL        => 42,
+    CAA        => 257,
+    CDNSKEY    => 60,
+    CDS        => 59,
+    CERT       => 37,
+    CNAME      => 5,
+    DHCID      => 49,
+    DLV        => 32769,
+    DNAME      => 39,
+    DNSKEY     => 48,
+    DS         => 43,
+    HIP        => 55,
+    HINFO      => 13,
+    IPSECKEY   => 45,
+    KEY        => 25,
+    KX         => 36,
+    LOC        => 29,
+    MX         => 15,
+    NAPTR      => 35,
+    NS         => 2,
+    NSEC       => 47,
+    NSEC3      => 50,
     NSEC3PARAM => 51,
     OPENPGPKEY => 61,
-    PTR => 12,
-    RRSIG => 46,
-    RP => 17,
-    SIG => 24,
-    SMIMEA => 53,
-    SOA => 6,
-    SRV => 33,
-    SSHFP => 44,
-    TA => 32768,
-    TKEY => 249,
-    TLSA => 52,
-    TSIG => 250,
-    TXT => 16,
-    URI => 256,
+    PTR        => 12,
+    RRSIG      => 46,
+    RP         => 17,
+    SIG        => 24,
+    SMIMEA     => 53,
+    SOA        => 6,
+    SRV        => 33,
+    SSHFP      => 44,
+    TA         => 32768,
+    TKEY       => 249,
+    TLSA       => 52,
+    TSIG       => 250,
+    TXT        => 16,
+    URI        => 256,
 };
 
 # Copied from libunbound
 use constant _ctx_err => {
-    -1 => 'socket error',
-    -2 => 'alloc failure',
-    -3 => 'syntax error',
-    -4 => 'DNS service failed',
-    -5 => 'fork() failed',
-    -6 => 'cfg change after finalize()',
-    -7 => 'initialization failed (bad settings)',
-    -8 => 'error in pipe communication with async bg worker',
-    -9 => 'error reading from file',
+    -1  => 'socket error',
+    -2  => 'alloc failure',
+    -3  => 'syntax error',
+    -4  => 'DNS service failed',
+    -5  => 'fork() failed',
+    -6  => 'cfg change after finalize()',
+    -7  => 'initialization failed (bad settings)',
+    -8  => 'error in pipe communication with async bg worker',
+    -9  => 'error reading from file',
     -10 => 'async_id does not exist or result already been delivered',
 };
 
@@ -157,7 +157,7 @@ To decode some common record types, see L</CONVENIENCE FUNCTIONS> below.
 
 Also B<NOTE:> libunbound doesn’t seem to offer effective controls for
 timing out a synchronous query.
-If timeouts are relevant for you, you probably want
+If timeouts are relevant for you, you probably need
 to use C<resolve_async()> instead.
 
 =cut
@@ -169,12 +169,18 @@ sub resolve {
     my $result = _resolve( $_[0]->{'_ub'}, $_[1], $type, $_[3] || () );
 
     if (!ref($result)) {
-        die DNS::Unbound::X->create('ResolveError', number => $result, string => _ub_strerror($result));
+        die _create_resolve_error($result);
     }
 
     return $result;
 }
 
+sub _create_resolve_error {
+    my ($number) = @_;
+
+    return DNS::Unbound::X->create('ResolveError', number => $number, string => _ub_strerror($number));
+}
+
 #----------------------------------------------------------------------
 
 =head2 $query = I<OBJ>->resolve_async( $NAME, $TYPE [, $CLASS ] );
@@ -182,11 +188,10 @@ sub resolve {
 Like C<resolve()> but starts an asynchronous query rather than a
 synchronous one.
 
-This returns an instance of C<DNS::Unbound::AsyncQuery>, which
-subclasses L<Promise::ES6>. You may C<cancel()> this promise object.
-The promise resolves with either the same hash reference as
-C<resolve()> returns, or it rejects with a L<DNS::Unbound::X> instance
-that describes the failure.
+This returns an instance of L<DNS::Unbound::AsyncQuery>.
+
+L<See below|/"METHODS FOR DEALING WITH ASYNCHRONOUS QUERIES"> for
+the methods you’ll need to use in tandem with this one.
 
 =cut
 
@@ -201,11 +206,15 @@ sub resolve_async {
 
     my ($res, $rej);
 
+    _load_asyncquery_if_needed();
+
     my $query = DNS::Unbound::AsyncQuery->new( sub {
         ($res, $rej) = @_;
     } );
 
     # This hash maintains the query state across all related promise objects.
+    # It must NOT contain $self, or else we’ll have a circular reference
+    # that will prevent this class’s DESTROY method from firing.
     my %dns = (
         res => $res,
         rej => $rej,
@@ -223,7 +232,7 @@ sub resolve_async {
     );
 
     if (my $err = $async_ar->[0]) {
-        die DNS::Unbound::X->create('ResolveError', number => $err, string => _ub_strerror($err));
+        die _create_resolve_error($err);
     }
 
     my $query_id = $async_ar->[1];
@@ -237,6 +246,17 @@ sub resolve_async {
     return $query;
 }
 
+sub _load_asyncquery_if_needed {
+    if (!$INC{'DNS/Unbound/AsyncQuery.pm'}) {
+        local ($@, $!);
+        require DNS::Unbound::AsyncQuery;
+
+        $DNS::Unbound::AsyncQuery::CANCEL_CR = \&DNS::Unbound::_ub_cancel;
+    }
+
+    return;
+}
+
 #----------------------------------------------------------------------
 
 =head2 I<OBJ>->enable_threads()
@@ -287,6 +307,12 @@ sub enable_threads {
 
 Sets a configuration option. Returns I<OBJ>.
 
+Note that this is basically just a passthrough to the underlying
+C<ub_ctx_set_option()> function and is thus subject to the same limitations
+as that function; for example, you can’t set C<verbosity> after the
+configuration has been “finalized”. (So use C<debuglevel()> for that
+instead.)
+
 =cut
 
 sub set_option {
@@ -324,7 +350,7 @@ sub get_option {
 Sets the debug level (an integer). Returns I<OBJ>.
 
 As of libunbound v1.9.2, this is just a way to set the C<verbosity>
-option after configuration is finalized.
+option regardless of whether the configuration is finalized.
 
 =cut
 
@@ -368,7 +394,7 @@ Gives the libunbound version string.
 
 =head1 METHODS FOR DEALING WITH ASYNCHRONOUS QUERIES
 
-The following methods correspond to their equivalents in libunbound:
+The following methods correspond to their equivalents in libunbound.
 
 =head2 I<OBJ>->poll()
 
@@ -438,7 +464,7 @@ sub _check_promises {
                 else {
                     $key = 'rej';
 
-                    $dns_hr->{'value'} = DNS::Unbound::X->create('ResolveError', number => $dns_hr->{'value'}, string => _ub_strerror($dns_hr->{'value'}));
+                    $dns_hr->{'value'} = _create_resolve_error($dns_hr->{'value'});
                 }
 
                 $dns_hr->{'fulfilled'} ||= do {
@@ -454,10 +480,22 @@ sub _check_promises {
 
 #----------------------------------------------------------------------
 
+# In case I decide to expose these:
+#
+# sub decode_ipv4 {
+#     shift if (ref $_[0]) && (ref $_[0])->isa(__PACKAGE__);
+#     return join('.', unpack('C4', $_[0]));
+# }
+#
+# sub decode_ipv6 {
+#     shift if (ref $_[0]) && (ref $_[0])->isa(__PACKAGE__);
+#     return join(':', unpack('(H4)8', $_[0]));
+# }
+
 =head1 CONVENIENCE FUNCTIONS
 
 Note that L<Socket> provides the C<inet_ntoa()> and C<inet_ntop()>
-functions for decoding C<A> and C<AAAA> records.
+functions for decoding the values of C<A> and C<AAAA> records.
 
 The following may be called either as object methods or as static
 functions (but not as class methods):
@@ -466,8 +504,9 @@ functions (but not as class methods):
 
 Decodes a DNS name. Useful for, e.g., C<NS> query results.
 
-Note that this will normally include a trailing C<.> because of the
-trailing NUL byte in an encoded DNS name.
+Note that this function’s return will normally include a trailing C<.>
+because of the trailing NUL byte in an encoded DNS name. This is normal
+and expected.
 
 =cut
 
@@ -505,55 +544,6 @@ sub DESTROY {
     };
 }
 
-#----------------------------------------------------------------------
-{
-    package DNS::Unbound::AsyncQuery;
-
-    use parent qw( Promise::ES6 );
-
-    sub then {
-        my $self = shift;
-
-        my $new = $self->SUPER::then(@_);
-
-        $new->{'_dns'} = $self->{'_dns'};
-
-        return $new;
-    }
-
-    sub cancel {
-        my ($self) = @_;
-
-        my $dns_hr = $self->{'_dns'};
-
-        if (!$dns_hr->{'fulfilled'}) {
-            if (my $ctx = delete $dns_hr->{'ctx'}) {
-                DNS::Unbound::_ub_cancel( $ctx, $dns_hr->{'id'} );
-            }
-        }
-
-        return;
-    }
-
-    # ----------------------------------------------------------------------
-    # Interfaces for DNS::Unbound to interact with the query’s DNS state:
-
-    sub _set_dns {
-        my ($self, $dns_hr) = @_;
-        $self->{'_dns'} = $dns_hr;
-        return $self;
-    }
-
-    sub _get_dns {
-        return $_[0]->{'_dns'};
-    }
-
-    sub _forget_dns {
-        delete $_[0]->{'_dns'};
-        return $_[0];
-    }
-}
-
 #----------------------------------------------------------------------
 
 1;
